Package: livelink
Title: Create Shareable Links for 'webR' and 'Shinylive' Environments
Version: 0.1.0
Authors@R: c(
    person("James Joseph", "Balamuta", , "james.balamuta@gmail.com", role = c("aut", "cre", "cph")),
    person("reprex authors", role = "cph",
           comment = "stringify_expression() and trim_common_leading_ws(), adapted in R/process-input.R; see inst/COPYRIGHTS")
  )
Description: Creates shareable links for 'R' code in 'WebAssembly' (WASM)
    Read-Eval-Print Loop (REPL) environments like 'webR'
    <https://webr.r-wasm.org/> and for 'Shiny' applications using
    'Shinylive' <https://shinylive.io/>. Supports single scripts,
    multi-file projects, exercise and solution pairs, and batch
    processing. Includes encoding, decoding, and previewing of links
    for both 'R' and 'Python' environments.
License: AGPL (>= 3)
URL: https://r-pkg.thecoatlessprofessor.com/livelink/,
        https://github.com/coatless-rpkg/livelink
BugReports: https://github.com/coatless-rpkg/livelink/issues
Depends: R (>= 4.1.0)
Imports: base64enc, cli, clipr, jsonlite, lzstring, RcppMsgPack, stats,
        tools, utils
Suggests: knitr, quarto, testthat (>= 3.0.0), withr
Encoding: UTF-8
SystemRequirements: Quarto command line tools
        (https://github.com/quarto-dev/quarto-cli).
Config/Needs/website: pkgdown
Config/testthat/edition: 3
Collate: 'as-data-frame.R' 'build-webr-repl-link.R' 'classes.R'
        'decode-shinylive-link.R' 'utils.R' 'decode-webr-repl-link.R'
        'extract-link.R' 'format-methods.R' 'knit-print.R'
        'knitr-engine.R' 'options.R' 'process-input.R'
        'shinylive-link.R' 'validators.R' 'webr-repl-directory-links.R'
        'webr-repl-links.R'
VignetteBuilder: quarto
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-17 08:50:52 UTC; ronin
Author: James Joseph Balamuta [aut, cre, cph],
  reprex authors [cph] (stringify_expression() and
    trim_common_leading_ws(), adapted in R/process-input.R; see
    inst/COPYRIGHTS)
Maintainer: James Joseph Balamuta <james.balamuta@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-24 10:20:02 UTC
