# Generated by roxygen2: do not edit by hand

S3method(as.character,shinylive_decoded)
S3method(as.character,shinylive_decoded_batch)
S3method(as.character,shinylive_directory)
S3method(as.character,shinylive_link)
S3method(as.character,shinylive_preview)
S3method(as.character,shinylive_project)
S3method(as.character,webr_decoded)
S3method(as.character,webr_decoded_batch)
S3method(as.character,webr_directory)
S3method(as.character,webr_exercise)
S3method(as.character,webr_link)
S3method(as.character,webr_preview)
S3method(as.character,webr_project)
S3method(as.data.frame,shinylive_decoded_batch)
S3method(as.data.frame,shinylive_directory)
S3method(as.data.frame,webr_decoded_batch)
S3method(as.data.frame,webr_directory)
S3method(format,shinylive_decoded)
S3method(format,shinylive_decoded_batch)
S3method(format,shinylive_directory)
S3method(format,shinylive_link)
S3method(format,shinylive_preview)
S3method(format,shinylive_project)
S3method(format,webr_decoded)
S3method(format,webr_decoded_batch)
S3method(format,webr_directory)
S3method(format,webr_exercise)
S3method(format,webr_link)
S3method(format,webr_preview)
S3method(format,webr_project)
S3method(knitr::knit_print,shinylive_directory)
S3method(knitr::knit_print,shinylive_link)
S3method(knitr::knit_print,shinylive_project)
S3method(knitr::knit_print,webr_directory)
S3method(knitr::knit_print,webr_exercise)
S3method(knitr::knit_print,webr_link)
S3method(knitr::knit_print,webr_project)
S3method(print,shinylive_decoded)
S3method(print,shinylive_decoded_batch)
S3method(print,shinylive_directory)
S3method(print,shinylive_link)
S3method(print,shinylive_preview)
S3method(print,shinylive_project)
S3method(print,webr_decoded)
S3method(print,webr_decoded_batch)
S3method(print,webr_directory)
S3method(print,webr_exercise)
S3method(print,webr_link)
S3method(print,webr_preview)
S3method(print,webr_project)
S3method(repl_urls,default)
export(decode_shinylive_link)
export(decode_webr_link)
export(preview_shinylive_link)
export(preview_webr_link)
export(repl_urls)
export(set_webr_base_url)
export(shinylive_directory)
export(shinylive_project)
export(shinylive_py_link)
export(shinylive_r_link)
export(use_livelink_engine)
export(use_livelink_hook)
export(webr_repl_directory)
export(webr_repl_exercise)
export(webr_repl_link)
export(webr_repl_project)
