# Generated by data-raw/aspell.R. R CMD check --as-cran reads this file to
# accept the words in .aspell/words.rds during its spelling pass.
Rd_files <- vignettes <- R_files <- description <-
  list(encoding = "UTF-8", dictionaries = c("en_stats", "words"))
