# print.webr_preview honors show_content

    Code
      print(preview)
    Message
      
      -- webR Link Preview --
      
      URL:
      <https://webr.r-wasm.org/latest/#code=eJyLrlbKS8xNVbJSKk4uyiwo0QtS0lEqSCzJAIroZ%2BTnpuqXpybFlxanFukjKShJrSgBaUlNLkotic9NLMpOLVKw0VUwMVKqjQUAjrwblA%3D%3D&jz>
      
      Files: 1
      Total size: 19 bytes
      Version: "latest"
      Encoding: "jz"
      
      'script.R' (19 bytes)
      
      Use print(preview, show_content = TRUE) to see file contents

---

    Code
      print(preview, show_content = TRUE)
    Message
      
      -- webR Link Preview --
      
      URL:
      <https://webr.r-wasm.org/latest/#code=eJyLrlbKS8xNVbJSKk4uyiwo0QtS0lEqSCzJAIroZ%2BTnpuqXpybFlxanFukjKShJrSgBaUlNLkotic9NLMpOLVKw0VUwMVKqjQUAjrwblA%3D%3D&jz>
      
      Files: 1
      Total size: 19 bytes
      Version: "latest"
      Encoding: "jz"
      
      'script.R' (19 bytes)
        secret_marker <- 42

# print.shinylive_preview honors show_content

    Code
      print(preview)
    Message
      
      -- Shinylive R Preview --
      
      Source:
      <https://shinylive.io/r/editor/#code=NobwRAdghgtgpmAXGKAHVA6ASmANGAYwHsIAXOMpMAZzgICc5SB9GKegaznoAIAeALQ8ALACY8YUgE9UCZOQAepMAF8AukA>
      
      Files: 1
      Total size: 19 bytes
      Engine: "R"
      Mode: "editor"
      
      'app.R' (text, 19 bytes)
      
      Use print(preview, show_content = TRUE) to see file contents

---

    Code
      print(preview, show_content = TRUE)
    Message
      
      -- Shinylive R Preview --
      
      Source:
      <https://shinylive.io/r/editor/#code=NobwRAdghgtgpmAXGKAHVA6ASmANGAYwHsIAXOMpMAZzgICc5SB9GKegaznoAIAeALQ8ALACY8YUgE9UCZOQAepMAF8AukA>
      
      Files: 1
      Total size: 19 bytes
      Engine: "R"
      Mode: "editor"
      
      'app.R' (text, 19 bytes)
        secret_marker <- 42

